@@ -88,7 +88,7 @@ try_to_aliases (const char **to_aliases,
   return FALSE;
 }
 
-extern const char **_g_charset_get_aliases (const char *canonical_name) G_GNUC_INTERNAL;
+extern const char ** G_GNUC_INTERNAL _g_charset_get_aliases (const char *canonical_name);
 
 /**
  * g_iconv_open:
