@@ -36,9 +36,8 @@
 /* uncomment the next line to get poll() debugging info */
 /* #define G_MAIN_POLL_DEBUG */
 
-#include "galias.h"
 #include "glib.h"
-#include "gthreadinit.h"
+#include "gthreadprivate.h"
 #include <signal.h>
 #include <sys/types.h>
 #include <time.h>
@@ -69,13 +68,17 @@
 #endif /* G_OS_WIN32 */
 
 #ifdef G_OS_BEOS
-#include <net/socket.h>
+#include <sys/socket.h>
+#include <sys/wait.h>
 #endif /* G_OS_BEOS */
 
 #ifdef G_OS_UNIX
 #include <fcntl.h>
 #include <sys/wait.h>
 #endif
+
+#include "galias.h"
+
 /* Types */
 
 typedef struct _GTimeoutSource GTimeoutSource;
@@ -99,6 +102,14 @@ struct _GMainWaiter
 };
 #endif  
 
+typedef struct _GMainDispatch GMainDispatch;
+
+struct _GMainDispatch
+{
+  gint depth;
+  GSList *source; /* stack of current sources */
+};
+
 struct _GMainContext
 {
 #ifdef G_THREADS_ENABLED
@@ -122,8 +133,6 @@ struct _GMainContext
   gint in_check_or_prepare;
 
   GPollRec *poll_records;
-  GPollRec *poll_free_list;
-  GMemChunk *poll_chunk;
   guint n_poll_records;
   GPollFD *cached_poll_array;
   guint cached_poll_array_size;
@@ -187,9 +196,9 @@ struct _GChildWatchSource
 
 struct _GPollRec
 {
-  gint priority;
   GPollFD *fd;
   GPollRec *next;
+  gint priority;
 };
 
 #ifdef G_THREADS_ENABLED
@@ -476,26 +485,17 @@ g_poll (GPollFD *fds,
 		f->revents |= G_IO_IN;
 	  }
     }
-#if 1 /* TEST_WITHOUT_THIS */
   else if (ready >= WAIT_OBJECT_0 && ready < WAIT_OBJECT_0 + nhandles)
     for (f = fds; f < &fds[nfds]; ++f)
       {
-	if ((f->events & (G_IO_IN | G_IO_OUT))
-	    && f->fd == (gint) handles[ready - WAIT_OBJECT_0])
+	if (f->fd == (gint) handles[ready - WAIT_OBJECT_0])
 	  {
-	    if (f->events & G_IO_IN)
-	      f->revents |= G_IO_IN;
-	    else
-	      f->revents |= G_IO_OUT;
+	    f->revents = f->events;
 #ifdef G_MAIN_POLL_DEBUG
 	    g_print ("g_poll: got event %#x\n", f->fd);
-#endif
-#if 0
-	    ResetEvent ((HANDLE) f->fd);
 #endif
 	  }
       }
-#endif
     
   return 1;
 }
@@ -603,25 +603,12 @@ g_main_context_ref (GMainContext *context)
   return context;
 }
 
-/* If DISABLE_MEM_POOLS is defined, then freeing the
- * mem chunk won't free the records, so we have to
- * do it manually. The conditionalization here is
- * an optimization; g_mem_chunk_free() is a no-op
- * when DISABLE_MEM_POOLS is set.
- */
-#ifdef DISABLE_MEM_POOLS
-static void
+static inline void
 poll_rec_list_free (GMainContext *context,
 		    GPollRec     *list)
 {
-  while (list)
-    {
-      GPollRec *tmp_rec = list;
-      list = list->next;
-      g_chunk_free (tmp_rec, context->poll_chunk);
-    }
+  g_slice_free_chain (GPollRec, list, next);
 }
-#endif /* DISABLE_MEM_POOLS */
 
 /**
  * g_main_context_unref:
@@ -648,7 +635,7 @@ g_main_context_unref (GMainContext *context)
   while (source)
     {
       GSource *next = source->next;
-      g_source_destroy_internal (source, context, TRUE);
+      g_source_destroy_internal (source, context, FALSE);
       source = next;
     }
 
@@ -659,14 +646,8 @@ g_main_context_unref (GMainContext *context)
   g_ptr_array_free (context->pending_dispatches, TRUE);
   g_free (context->cached_poll_array);
 
-#ifdef DISABLE_MEM_POLLS
   poll_rec_list_free (context, context->poll_records);
-  poll_rec_list_free (context, context->poll_free_list);
-#endif /* DISABLE_MEM_POOLS */
   
-  if (context->poll_chunk) 
-    g_mem_chunk_destroy (context->poll_chunk);
-
 #ifdef G_THREADS_ENABLED
   if (g_thread_supported())
     {
@@ -907,7 +888,8 @@ g_source_list_remove (GSource      *source,
  * Adds a #GSource to a @context so that it will be executed within
  * that context.
  *
- * Return value: the ID for the source within the #GMainContext
+ * Return value: the ID (greater than 0) for the source within the 
+ *   #GMainContext. 
  **/
 guint
 g_source_attach (GSource      *source,
@@ -1021,10 +1003,11 @@ g_source_destroy (GSource *source)
  * @source: a #GSource
  * 
  * Returns the numeric ID for a particular source. The ID of a source
- * is unique within a particular main loop context. The reverse
+ * is a positive integer which is unique within a particular main loop 
+ * context. The reverse
  * mapping from ID to source is done by g_main_context_find_source_by_id().
  *
- * Return value: the ID for the source
+ * Return value: the ID (greater than 0) for the source
  **/
 guint
 g_source_get_id (GSource *source)
@@ -1252,6 +1235,29 @@ g_source_set_callback (GSource        *source,
   g_source_set_callback_indirect (source, new_callback, &g_source_callback_funcs);
 }
 
+
+/**
+ * g_source_set_funcs:
+ * @source: a #GSource
+ * @funcs: the new #GSourceFuncs
+ * 
+ * Sets the source functions (can be used to override 
+ * default implementations) of an unattached source.
+ * 
+ * Since: 2.12
+ */
+void
+g_source_set_funcs (GSource     *source,
+	           GSourceFuncs *funcs)
+{
+  g_return_if_fail (source != NULL);
+  g_return_if_fail (source->context == NULL);
+  g_return_if_fail (source->ref_count > 0);
+  g_return_if_fail (funcs != NULL);
+
+  source->source_funcs = funcs;
+}
+
 /**
  * g_source_set_priority:
  * @source: a #GSource
@@ -1468,9 +1474,9 @@ g_source_unref (GSource *source)
 /**
  * g_main_context_find_source_by_id:
  * @context: a #GMainContext (if %NULL, the default context will be used)
- * @source_id: the source ID, as returned by g_source_get_id()
+ * @source_id: the source ID, as returned by g_source_get_id(). 
  * 
- * Finds a #GSource given a pair of context and ID
+ * Finds a #GSource given a pair of context and ID.
  * 
  * Return value: the #GSource if found, otherwise, %NULL
  **/
@@ -1596,9 +1602,10 @@ g_main_context_find_source_by_user_data (GMainContext *context,
 
 /**
  * g_source_remove:
- * @tag: the id of the source to remove.
+ * @tag: the ID of the source to remove.
  * 
- * Removes the source with the given id from the default main context. The id of
+ * Removes the source with the given id from the default main context. 
+ * The id of
  * a #GSource is given by g_source_get_id(), or will be returned by the
  * functions g_source_attach(), g_idle_add(), g_idle_add_full(),
  * g_timeout_add(), g_timeout_add_full(), g_child_watch_add(),
@@ -1696,44 +1703,42 @@ g_get_current_time (GTimeVal *result)
   result->tv_sec = r.tv_sec;
   result->tv_usec = r.tv_usec;
 #else
-  /* Avoid calling time() except for the first time.
-   * GetTickCount() should be pretty fast and low-level?
-   * I could also use ftime() but it seems unnecessarily overheady.
+  FILETIME ft;
+  guint64 *time64 = (guint64 *) &ft;
+
+  GetSystemTimeAsFileTime (&ft);
+
+  /* Convert from 100s of nanoseconds since 1601-01-01
+   * to Unix epoch. Yes, this is Y2038 unsafe.
    */
-  static DWORD start_tick = 0;
-  static time_t start_time;
-  DWORD tick;
+  *time64 -= G_GINT64_CONSTANT (116444736000000000);
+  *time64 /= 10;
 
-  g_return_if_fail (result != NULL);
- 
-  if (start_tick == 0)
-    {
-      start_tick = GetTickCount ();
-      time (&start_time);
-    }
-
-  tick = GetTickCount ();
-
-  result->tv_sec = (tick - start_tick) / 1000 + start_time;
-  result->tv_usec = ((tick - start_tick) % 1000) * 1000;
+  result->tv_sec = *time64 / 1000000;
+  result->tv_usec = *time64 % 1000000;
 #endif
 }
 
+static void
+g_main_dispatch_free (gpointer dispatch)
+{
+  g_slice_free (GMainDispatch, dispatch);
+}
+
 /* Running the main loop */
 
-static gint *
-get_depth_pointer (void)
+static GMainDispatch *
+get_dispatch (void)
 {
   static GStaticPrivate depth_private = G_STATIC_PRIVATE_INIT;
-  gint *depth_pointer = g_static_private_get (&depth_private);
-  if (!depth_pointer)
+  GMainDispatch *dispatch = g_static_private_get (&depth_private);
+  if (!dispatch)
     {
-      depth_pointer = g_new (gint, 1);
-      *depth_pointer = 0;
-      g_static_private_set (&depth_private, depth_pointer, g_free);
+      dispatch = g_slice_new0 (GMainDispatch);
+      g_static_private_set (&depth_private, dispatch, g_main_dispatch_free);
     }
 
-  return depth_pointer;
+  return dispatch;
 }
 
 /**
@@ -1855,10 +1860,100 @@ get_depth_pointer (void)
 int
 g_main_depth (void)
 {
-  gint *depth = get_depth_pointer ();
-  return *depth;
+  GMainDispatch *dispatch = get_dispatch ();
+  return dispatch->depth;
 }
 
+/**
+ * g_main_current_source:
+ *
+ * Returns the currently firing source for this thread.
+ * 
+ * Return value: The currently firing source or %NULL.
+ *
+ * Since: 2.12
+ */
+GSource *
+g_main_current_source (void)
+{
+  GMainDispatch *dispatch = get_dispatch ();
+  return dispatch->source ? dispatch->source->data : NULL;
+}
+
+/**
+ * g_source_is_destroyed:
+ * @source: a #GSource
+ *
+ * Returns whether @source has been destroyed.
+ *
+ * This is important when you operate upon your objects 
+ * from within idle handlers, but may have freed the object 
+ * before the dispatch of your idle handler.
+ *
+ * <informalexample><programlisting>
+ * static gboolean 
+ * idle_callback (gpointer data)
+ * {
+ *   SomeWidget *self = data;
+ *    
+ *   GDK_THREADS_ENTER (<!-- -->);
+ *   /<!-- -->* do stuff with self *<!-- -->/
+ *   self->idle_id = 0;
+ *   GDK_THREADS_LEAVE (<!-- -->);
+ *    
+ *   return FALSE;
+ * }
+ *  
+ * static void 
+ * some_widget_do_stuff_later (SomeWidget *self)
+ * {
+ *   self->idle_id = g_idle_add (idle_callback, self);
+ * }
+ *  
+ * static void 
+ * some_widget_finalize (GObject *object)
+ * {
+ *   SomeWidget *self = SOME_WIDGET (object);
+ *    
+ *   if (self->idle_id)
+ *     g_source_remove (self->idle_id);
+ *    
+ *   G_OBJECT_CLASS (parent_class)->finalize (object);
+ * }
+ * </programlisting></informalexample>
+ *
+ * This will fail in a multi-threaded application if the 
+ * widget is destroyed before the idle handler fires due 
+ * to the use after free in the callback. A solution, to 
+ * this particular problem, is to check to if the source
+ * has already been destroy within the callback.
+ *
+ * <informalexample><programlisting>
+ * static gboolean 
+ * idle_callback (gpointer data)
+ * {
+ *   SomeWidget *self = data;
+ *   
+ *   GDK_THREADS_ENTER ();
+ *   if (!g_source_is_destroyed (g_main_current_source ()))
+ *     {
+ *       /<!-- -->* do stuff with self *<!-- -->/
+ *     }
+ *   GDK_THREADS_LEAVE ();
+ *   
+ *   return FALSE;
+ * }
+ * </programlisting></informalexample>
+ *
+ * Return value: %TRUE if the source has been destroyed
+ */
+gboolean
+g_source_is_destroyed (GSource *source)
+{
+  return SOURCE_DESTROYED (source);
+}
+
+
 /* Temporarily remove all this source's file descriptors from the
  * poll(), so that if data comes available for one of the file descriptors
  * we don't continually spin in the poll()
@@ -1900,7 +1995,7 @@ unblock_source (GSource *source)
 static void
 g_main_dispatch (GMainContext *context)
 {
-  gint *depth = get_depth_pointer ();
+  GMainDispatch *current = get_dispatch ();
   guint i;
 
   for (i = 0; i < context->pending_dispatches->len; i++)
@@ -1943,18 +2038,20 @@ g_main_dispatch (GMainContext *context)
 
 	  UNLOCK_CONTEXT (context);
 
-	  (*depth)++;
+	  current->depth++;
+	  current->source = g_slist_prepend (current->source, source);
 	  need_destroy = ! dispatch (source,
 				     callback,
 				     user_data);
-	  (*depth)--;
+	  current->source = g_slist_remove (current->source, source);
+	  current->depth--;
 	  
- 	  LOCK_CONTEXT (context);
-
 	  if (cb_funcs)
 	    cb_funcs->unref (cb_data);
 
-	 if (!was_in_call)
+ 	  LOCK_CONTEXT (context);
+	  
+	  if (!was_in_call)
 	    source->flags &= ~G_HOOK_FLAG_IN_CALL;
 
 	  if ((source->flags & G_SOURCE_CAN_RECURSE) == 0 &&
@@ -2964,18 +3061,8 @@ g_main_context_add_poll_unlocked (GMainContext *context,
 				  gint          priority,
 				  GPollFD      *fd)
 {
-  GPollRec *lastrec, *pollrec, *newrec;
-
-  if (!context->poll_chunk)
-    context->poll_chunk = g_mem_chunk_create (GPollRec, 32, G_ALLOC_ONLY);
-
-  if (context->poll_free_list)
-    {
-      newrec = context->poll_free_list;
-      context->poll_free_list = newrec->next;
-    }
-  else
-    newrec = g_chunk_new (GPollRec, context->poll_chunk);
+  GPollRec *lastrec, *pollrec;
+  GPollRec *newrec = g_slice_new (GPollRec);
 
   /* This file descriptor may be checked before we ever poll */
   fd->revents = 0;
@@ -3048,12 +3135,7 @@ g_main_context_remove_poll_unlocked (GMainContext *context,
 	  else
 	    context->poll_records = pollrec->next;
 
-#ifdef ENABLE_GC_FRIENDLY
-	  pollrec->fd = NULL;  
-#endif /* ENABLE_GC_FRIENDLY */
-
-	  pollrec->next = context->poll_free_list;
-	  context->poll_free_list = pollrec;
+	  g_slice_free (GPollRec, pollrec);
 
 	  context->n_poll_records--;
 	  break;
@@ -3205,6 +3287,38 @@ g_main_context_wakeup (GMainContext *context)
   UNLOCK_CONTEXT (context);
 }
 
+/**
+ * g_main_context_is_owner:
+ * @context: a #GMainContext
+ * 
+ * Determines whether this thread holds the (recursive)
+ * ownership of this #GMaincontext. This is useful to
+ * know before waiting on another thread that may be
+ * blocking to get ownership of @context.
+ *
+ * Returns: %TRUE if current thread is owner of @context.
+ *
+ * Since: 2.10
+ **/
+gboolean
+g_main_context_is_owner (GMainContext *context)
+{
+  gboolean is_owner;
+
+  if (!context)
+    context = g_main_context_default ();
+
+#ifdef G_THREADS_ENABLED
+  LOCK_CONTEXT (context);
+  is_owner = context->owner == G_THREAD_SELF;
+  UNLOCK_CONTEXT (context);
+#else
+  is_owner = TRUE;
+#endif
+
+  return is_owner;
+}
+
 /* Timeouts */
 
 static void
@@ -3366,7 +3480,7 @@ g_timeout_source_new (guint interval)
  * timeout is recalculated based on the current time and the given interval
  * (it does not try to 'catch up' time lost in delays).
  * 
- * Return value: the id of event source.
+ * Return value: the ID (greater than 0) of the event source.
  **/
 guint
 g_timeout_add_full (gint           priority,
@@ -3411,7 +3525,7 @@ g_timeout_add_full (gint           priority,
  * timeout is recalculated based on the current time and the given interval
  * (it does not try to 'catch up' time lost in delays).
  * 
- * Return value: the id of event source.
+ * Return value: the ID (greater than 0) of the event source.
  **/
 guint 
 g_timeout_add (guint32        interval,
@@ -3505,11 +3619,8 @@ static gboolean
 g_child_watch_prepare (GSource *source,
 		       gint    *timeout)
 {
-  GChildWatchSource *child_watch_source;
   *timeout = -1;
 
-  child_watch_source = (GChildWatchSource *) source;
-
   return check_for_child_exited (source);
 }
 
@@ -3517,10 +3628,6 @@ g_child_watch_prepare (GSource *source,
 static gboolean 
 g_child_watch_check (GSource  *source)
 {
-  GChildWatchSource *child_watch_source;
-
-  child_watch_source = (GChildWatchSource *) source;
-
   return check_for_child_exited (source);
 }
 
@@ -3579,25 +3686,13 @@ g_child_watch_source_init_single (void)
 
   action.sa_handler = g_child_watch_signal_handler;
   sigemptyset (&action.sa_mask);
-  action.sa_flags = SA_RESTART | SA_NOCLDSTOP;
+  action.sa_flags = SA_NOCLDSTOP;
   sigaction (SIGCHLD, &action, NULL);
 }
 
 static gpointer
 child_watch_helper_thread (gpointer data)
 {
-  GPollFD fds;
-  GPollFunc poll_func;
-
-#ifdef HAVE_POLL
-      poll_func = (GPollFunc)poll;
-#else
-      poll_func = g_poll;
-#endif
-
-  fds.fd = child_watch_wake_up_pipe[0];
-  fds.events = G_IO_IN;
-
   while (1)
     {
       gchar b[20];
@@ -3621,7 +3716,6 @@ child_watch_helper_thread (gpointer data)
 	}
       G_UNLOCK (main_context_list);
     }
-  return NULL;
 }
 
 static void
@@ -3684,10 +3778,19 @@ g_child_watch_source_init (void)
  * and must be added to one with g_source_attach() before it will be
  * executed.
  * 
+ * Note that child watch sources can only be used in conjunction with
+ * <literal>g_spawn...</literal> when the %G_SPAWN_DO_NOT_REAP_CHILD
+ * flag is used.
+ *
  * Note that on platforms where #GPid must be explicitely closed
  * (see g_spawn_close_pid()) @pid must not be closed while the
  * source is still active. Typically, you will want to call
  * g_spawn_close_pid() in the callback function for the source.
+ *
+ * Note further that using g_child_watch_source_new() is not 
+ * compatible with calling <literal>waitpid(-1)</literal> in 
+ * the application. Calling waitpid() for individual pids will
+ * still work fine. 
  * 
  * Return value: the newly-created child watch source
  *
@@ -3732,7 +3835,7 @@ g_child_watch_source_new (GPid pid)
  * 
  * GLib supports only a single callback per process id.
  *
- * Return value: the id of event source.
+ * Return value: the ID (greater than 0) of the event source.
  *
  * Since: 2.4
  **/
@@ -3776,7 +3879,7 @@ g_child_watch_add_full (gint            priority,
  *
  * GLib supports only a single callback per process id.
  *
- * Return value: the id of event source.
+ * Return value: the ID (greater than 0) of the event source.
  *
  * Since: 2.4
  **/
@@ -3857,7 +3960,7 @@ g_idle_source_new (void)
  * events pending.  If the function returns %FALSE it is automatically
  * removed from the list of event sources and will not be called again.
  * 
- * Return value: the id of the event source.
+ * Return value: the ID (greater than 0) of the event source.
  **/
 guint 
 g_idle_add_full (gint           priority,
@@ -3893,7 +3996,7 @@ g_idle_add_full (gint           priority,
  * returns %FALSE it is automatically removed from the list of event
  * sources and will not be called again.
  * 
- * Return value: the id of the event source.
+ * Return value: the ID (greater than 0) of the event source.
  **/
 guint 
 g_idle_add (GSourceFunc    function,
@@ -3916,3 +4019,5 @@ g_idle_remove_by_data (gpointer data)
   return g_source_remove_by_funcs_user_data (&g_idle_funcs, data);
 }
 
+#define __G_MAIN_C__
+#include "galiasdef.c"
