@@ -405,9 +405,8 @@ vasnprintf (char *resultbuf, size_t *lengthp, const char *format, va_list args)
 			  const char *digitp = dp->precision_start + 1;
 
 			  precision = 0;
-			  do
+			  while (digitp != dp->precision_end)
 			    precision = precision * 10 + (*digitp++ - '0');
-			  while (digitp != dp->precision_end);
 			}
 		    }
 
@@ -615,11 +614,28 @@ vasnprintf (char *resultbuf, size_t *lengthp, const char *format, va_list args)
 
 		switch (type)
 		  {
+#ifdef HAVE_INT64_AND_I64
+		  case TYPE_INT64:
+		  case TYPE_UINT64:
+		    *p++ = 'I';
+		    *p++ = '6';
+		    *p++ = '4';
+		    break;
+#endif
 #ifdef HAVE_LONG_LONG
 		  case TYPE_LONGLONGINT:
 		  case TYPE_ULONGLONGINT:
+#ifdef HAVE_INT64_AND_I64	/* The system (sn)printf uses %I64. Also assume
+				 * that long long == __int64.
+				 */
+		    *p++ = 'I';
+		    *p++ = '6';
+		    *p++ = '4';
+		    break;
+#else
 		    *p++ = 'l';
 		    /*FALLTHROUGH*/
+#endif
 #endif
 		  case TYPE_LONGINT:
 		  case TYPE_ULONGINT:
@@ -769,6 +785,20 @@ vasnprintf (char *resultbuf, size_t *lengthp, const char *format, va_list args)
 			  SNPRINTF_BUF (arg);
 			}
 			break;
+#ifdef HAVE_INT64_AND_I64
+		      case TYPE_INT64:
+			{
+			  __int64 arg = a.arg[dp->arg_index].a.a_int64;
+			  SNPRINTF_BUF (arg);
+			}
+			break;
+		      case TYPE_UINT64:
+			{
+			  unsigned __int64 arg = a.arg[dp->arg_index].a.a_uint64;
+			  SNPRINTF_BUF (arg);
+			}
+			break;			
+#endif
 #ifdef HAVE_LONG_LONG
 #ifndef HAVE_LONG_LONG_FORMAT
 		      case TYPE_LONGLONGINT:
@@ -823,11 +853,19 @@ vasnprintf (char *resultbuf, size_t *lengthp, const char *format, va_list args)
 				}
 			    }
 			  
-			  count = print_long_long (result + length, maxlen,
+#if HAVE_SNPRINTF
+ 			  count = print_long_long (result + length, maxlen,
+ 						   width, precision,
+ 						   dp->flags,
+ 						   dp->conversion,
+ 						   arg);
+#else
+			  count = print_long_long (tmp, tmp_length,
 						   width, precision,
 						   dp->flags,
 						   dp->conversion,
 						   arg);
+#endif
 			}
 			break;
 #else
