@@ -40,17 +40,6 @@
 
 #include "glibintl.h"
 
-/* With solaris threads, fork() duplicates all threads, which
- * a) could cause unexpected side-effects, and b) is expensive.
- * Once we remove support for solaris threads, the FORK1 #define
- * should be removedl
- */
-#ifdef G_THREADS_IMPL_SOLARIS
-#define FORK1() fork1()
-#else
-#define FORK1() fork()
-#endif
-
 static gint g_execute (const gchar  *file,
                        gchar **argv,
                        gchar **envp,
@@ -79,10 +68,7 @@ static gboolean fork_exec_with_pipes (gboolean              intermediate_child,
 GQuark
 g_spawn_error_quark (void)
 {
-  static GQuark quark = 0;
-  if (quark == 0)
-    quark = g_quark_from_static_string ("g-exec-error-quark");
-  return quark;
+  return g_quark_from_static_string ("g-exec-error-quark");
 }
 
 /**
@@ -1094,7 +1080,7 @@ fork_exec_with_pipes (gboolean              intermediate_child,
   if (standard_error && !make_pipe (stderr_pipe, error))
     goto cleanup_and_fail;
 
-  pid = FORK1 ();
+  pid = fork ();
 
   if (pid < 0)
     {      
@@ -1136,7 +1122,7 @@ fork_exec_with_pipes (gboolean              intermediate_child,
            */
           GPid grandchild_pid;
 
-          grandchild_pid = FORK1 ();
+          grandchild_pid = fork ();
 
           if (grandchild_pid < 0)
             {
