@@ -24,6 +24,8 @@
 #include	"gclosure.h"
 #include	"gvaluecollector.h"
 
+#include	"gobjectalias.h"
+
 #include <string.h>
 
 /* --- typedefs & structures --- */
@@ -41,7 +43,12 @@ static gint	boxed_nodes_cmp		(gconstpointer	p1,
 
 
 /* --- variables --- */
-static GBSearchArray boxed_bsa = G_STATIC_BSEARCH_ARRAY_INIT (sizeof (BoxedNode), boxed_nodes_cmp, 0);
+static GBSearchArray *boxed_bsa = NULL;
+static const GBSearchConfig boxed_bconfig = {
+  sizeof (BoxedNode),
+  boxed_nodes_cmp,
+  0,
+};
 
 
 /* --- functions --- */
@@ -86,24 +93,8 @@ value_free (gpointer boxed)
   g_free (value);
 }
 
-static gpointer
-gstring_copy (gpointer boxed)
-{
-  const GString *src_gstring = boxed;
-
-  return g_string_new_len (src_gstring->str, src_gstring->len);
-}
-
-static void
-gstring_free (gpointer boxed)
-{
-  GString *gstring = boxed;
-
-  g_string_free (gstring, TRUE);
-}
-
 void
-g_boxed_type_init (void)  /* sync with gtype.c */
+g_boxed_type_init (void) 
 {
   static const GTypeInfo info = {
     0,                          /* class_size */
@@ -120,9 +111,11 @@ g_boxed_type_init (void)  /* sync with gtype.c */
   const GTypeFundamentalInfo finfo = { G_TYPE_FLAG_DERIVABLE, };
   GType type;
 
+  boxed_bsa = g_bsearch_array_create (&boxed_bconfig);
+
   /* G_TYPE_BOXED
    */
-  type = g_type_register_fundamental (G_TYPE_BOXED, "GBoxed", &info, &finfo,
+  type = g_type_register_fundamental (G_TYPE_BOXED, g_intern_static_string ("GBoxed"), &info, &finfo,
 				      G_TYPE_FLAG_ABSTRACT | G_TYPE_FLAG_VALUE_ABSTRACT);
   g_assert (type == G_TYPE_BOXED);
 }
@@ -133,7 +126,7 @@ g_closure_get_type (void)
   static GType type_id = 0;
 
   if (!type_id)
-    type_id = g_boxed_type_register_static ("GClosure",
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GClosure"),
 					    (GBoxedCopyFunc) g_closure_ref,
 					    (GBoxedFreeFunc) g_closure_unref);
   return type_id;
@@ -145,7 +138,7 @@ g_value_get_type (void)
   static GType type_id = 0;
 
   if (!type_id)
-    type_id = g_boxed_type_register_static ("GValue",
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GValue"),
 					    value_copy,
 					    value_free);
   return type_id;
@@ -157,31 +150,100 @@ g_value_array_get_type (void)
   static GType type_id = 0;
 
   if (!type_id)
-    type_id = g_boxed_type_register_static ("GValueArray",
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GValueArray"),
 					    (GBoxedCopyFunc) g_value_array_copy,
 					    (GBoxedFreeFunc) g_value_array_free);
   return type_id;
 }
 
+static gpointer
+gdate_copy (gpointer boxed)
+{
+  const GDate *date = (const GDate*) boxed;
+
+  return g_date_new_julian (g_date_get_julian (date));
+}
+
+GType
+g_date_get_type (void)
+{
+  static GType type_id = 0;
+
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GDate"),
+					    (GBoxedCopyFunc) gdate_copy,
+					    (GBoxedFreeFunc) g_date_free);
+  return type_id;
+}
+
+GType
+g_strv_get_type (void)
+{
+  static GType type_id = 0;
+
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GStrv"),
+					    (GBoxedCopyFunc) g_strdupv,
+					    (GBoxedFreeFunc) g_strfreev);
+  return type_id;
+}
+
+static gpointer
+gstring_copy (gpointer boxed)
+{
+  const GString *src_gstring = boxed;
+
+  return g_string_new_len (src_gstring->str, src_gstring->len);
+}
+
+static void
+gstring_free (gpointer boxed)
+{
+  GString *gstring = boxed;
+
+  g_string_free (gstring, TRUE);
+}
+
 GType
 g_gstring_get_type (void)
 {
   static GType type_id = 0;
 
   if (!type_id)
-    type_id = g_boxed_type_register_static ("GString",	/* the naming is a bit odd, but GString is obviously not G_TYPE_STRING */
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GString"),
+                                            /* the naming is a bit odd, but GString is obviously not G_TYPE_STRING */
 					    gstring_copy,
 					    gstring_free);
   return type_id;
 }
 
+static gpointer
+hash_table_copy (gpointer boxed)
+{
+  GHashTable *hash_table = boxed;
+  return g_hash_table_ref (hash_table);
+}
+
+static void
+hash_table_free (gpointer boxed)
+{
+  GHashTable *hash_table = boxed;
+  g_hash_table_unref (hash_table);
+}
+
+GType
+g_hash_table_get_type (void)
+{
+  static GType type_id = 0;
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GHashTable"),
+					    hash_table_copy, hash_table_free);
+  return type_id;
+}
+
 static void
 boxed_proxy_value_init (GValue *value)
 {
-  BoxedNode key, *node;
-
-  key.type = G_VALUE_TYPE (value);
-  node = g_bsearch_array_lookup (&boxed_bsa, &key);
   value->data[0].v_pointer = NULL;
 }
 
@@ -193,7 +255,7 @@ boxed_proxy_value_free (GValue *value)
       BoxedNode key, *node;
 
       key.type = G_VALUE_TYPE (value);
-      node = g_bsearch_array_lookup (&boxed_bsa, &key);
+      node = g_bsearch_array_lookup (boxed_bsa, &boxed_bconfig, &key);
       node->free (value->data[0].v_pointer);
     }
 }
@@ -207,7 +269,7 @@ boxed_proxy_value_copy (const GValue *src_value,
       BoxedNode key, *node;
 
       key.type = G_VALUE_TYPE (src_value);
-      node = g_bsearch_array_lookup (&boxed_bsa, &key);
+      node = g_bsearch_array_lookup (boxed_bsa, &boxed_bconfig, &key);
       dest_value->data[0].v_pointer = node->copy (src_value->data[0].v_pointer);
     }
   else
@@ -229,7 +291,7 @@ boxed_proxy_collect_value (GValue      *value,
   BoxedNode key, *node;
 
   key.type = G_VALUE_TYPE (value);
-  node = g_bsearch_array_lookup (&boxed_bsa, &key);
+  node = g_bsearch_array_lookup (boxed_bsa, &boxed_bconfig, &key);
 
   if (!collect_values[0].v_pointer)
     value->data[0].v_pointer = NULL;
@@ -267,7 +329,7 @@ boxed_proxy_lcopy_value (const GValue *value,
       BoxedNode key, *node;
 
       key.type = G_VALUE_TYPE (value);
-      node = g_bsearch_array_lookup (&boxed_bsa, &key);
+      node = g_bsearch_array_lookup (boxed_bsa, &boxed_bconfig, &key);
       *boxed_p = node->copy (value->data[0].v_pointer);
     }
 
@@ -318,7 +380,7 @@ g_boxed_type_register_static (const gchar   *name,
       key.type = type;
       key.copy = boxed_copy;
       key.free = boxed_free;
-      g_bsearch_array_insert (&boxed_bsa, &key, TRUE);
+      boxed_bsa = g_bsearch_array_insert (boxed_bsa, &boxed_bconfig, &key);
     }
 
   return type;
@@ -345,14 +407,14 @@ g_boxed_copy (GType         boxed_type,
       BoxedNode key, *node;
 
       key.type = boxed_type;
-      node = g_bsearch_array_lookup (&boxed_bsa, &key);
+      node = g_bsearch_array_lookup (boxed_bsa, &boxed_bconfig, &key);
       dest_boxed = node->copy ((gpointer) src_boxed);
     }
   else
     {
       GValue src_value, dest_value;
       
-      /* we heavil rely on third-party boxed type value vtable
+      /* we heavily rely on third-party boxed type value vtable
        * implementations to follow normal boxed value storage
        * (data[0].v_pointer is the boxed struct, and
        * data[1].v_uint holds the G_VALUE_NOCOPY_CONTENTS flag,
@@ -367,14 +429,12 @@ g_boxed_copy (GType         boxed_type,
       src_value.data[0].v_pointer = (gpointer) src_boxed;
       src_value.data[1].v_uint = G_VALUE_NOCOPY_CONTENTS;
 
-      /* call third-party code copy fucntion, fingers-crossed */
+      /* call third-party code copy function, fingers-crossed */
       value_meminit (&dest_value, boxed_type);
       value_table->value_copy (&src_value, &dest_value);
 
       /* double check and grouse if things went wrong */
-      if (dest_value.data[1].v_ulong ||
-	  dest_value.data[2].v_ulong ||
-	  dest_value.data[3].v_ulong)
+      if (dest_value.data[1].v_ulong)
 	g_warning ("the copy_value() implementation of type `%s' seems to make use of reserved GValue fields",
 		   g_type_name (boxed_type));
 
@@ -404,7 +464,7 @@ g_boxed_free (GType    boxed_type,
       BoxedNode key, *node;
 
       key.type = boxed_type;
-      node = g_bsearch_array_lookup (&boxed_bsa, &key);
+      node = g_bsearch_array_lookup (boxed_bsa, &boxed_bconfig, &key);
       node->free (boxed);
     }
   else
@@ -457,7 +517,7 @@ value_set_boxed_internal (GValue       *value,
     }
 
   key.type = G_VALUE_TYPE (value);
-  node = g_bsearch_array_lookup (&boxed_bsa, &key);
+  node = g_bsearch_array_lookup (boxed_bsa, &boxed_bconfig, &key);
 
   if (node)
     {
@@ -502,9 +562,19 @@ g_value_set_static_boxed (GValue       *value,
 void
 g_value_set_boxed_take_ownership (GValue       *value,
 				  gconstpointer boxed)
+{
+  g_value_take_boxed (value, boxed);
+}
+
+void
+g_value_take_boxed (GValue       *value,
+		    gconstpointer boxed)
 {
   g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
   g_return_if_fail (G_TYPE_IS_VALUE (G_VALUE_TYPE (value)));
 
   value_set_boxed_internal (value, boxed, FALSE, TRUE);
 }
+
+#define __G_BOXED_C__
+#include "gobjectaliasdef.c"
